;****************************************************************************
;*                   SAVESCR.ASM      Version 0.80
;*
;*
;*             Started: 890123        Timelog: 7h (890211)
;****************************************************************************
;* Programming I/F
;*
;*      AH=255, INT 1CH         Check if loaded.
;*                              AX=1999 if loaded
;*
;*      AH=254, INT 1CH         Set function on or off
;*                              AL=0 (off), AL=1 (on)
;*                              AX=1999 if ok
;*      AH=253, INT 1CH         Set blankout time
;*                              BX=new time (in 1/18 sec)
;*                              AX=1999 if ok
;*
;*
;****************************************************************************
code segment para public 'code'
	assume  cs:code
        org     100h

start:  jmp     init

old_kbdint      label   dword
old_kbdvec      dw      2 dup (?)       ;set aside a double word for old I/O
old_timerint    label   dword
old_timervec    dw      2 dup (?)       ;set aside a double word for old I/O

adapter         db      0
msr_address     dw      0
msr_values      db      2ch,28h,2dh,29h
		db      2ah,2eh,1eh,29h

switch          dw      00
isblank         db      00
blanktime       dw      300

vga_index       dw      00
vga_clock       dw      00

blank_on        db      00

;****************************************************************************
;* Keyboard interupt (INT 09H) comes here.                                  *
;*                                                                          *
;****************************************************************************
int_kbd         proc near
	push    ax                      ;save registers
        push    bx
        push    cx
        push    dx
        push    si
        push    di
        push    ds
	push    es

	mov     al,blank_on             ;is function on ?
	cmp     al,1
	jz      kbd_1
	jmp     short exit              ;no exit

kbd_1:  xor     ax,ax                   ;set switch to zero
	mov     switch,ax
	mov     al,isblank              ;check if screen is blank
	cmp     al,1
	jnz     exit

	call    video_enable

	xor     ax,ax                   ;set screen to not_blank
	mov     isblank,al
	pop     es                      ;restore registers
	pop     ds
	pop     di
	pop     si
	pop     dx
	pop     cx
	pop     bx
	pop     ax
	sti
	jmp     old_kbdint              ;jump to old INT 9

exit:   pop     es                      ;restore registers
        pop     ds
        pop     di
        pop     si
        pop     dx
        pop     cx
        pop     bx
	pop     ax
	sti
	jmp     old_kbdint              ;jump to old INT 9
int_kbd         endp

;****************************************************************************
;* Timer interupt (INT 1CH) comes here.                                     *
;*                                                                          *
;****************************************************************************
int_timer       proc near
	cmp     ah,0ffh                 ;do i want to check for prior load
	jnz     no_check1               ;no do timer
	mov     ax,1999                 ;yes set ax=1999
	sti
	iret                            ;dump timer irq

no_check1:
	cmp     ah,0feh                 ;is turn on/off requested
	jnz     no_check2                ;no, continue
	mov     blank_on,al             ;yes, set staus variable
	mov     al,isblank              ;check if screen is blank
	cmp     al,1
	jnz     set_ok
	call    video_enable
	xor     ax,ax                   ;set screen to not_blank
	mov     isblank,al
set_ok: xor     ax,ax                   ;set switch to zero
	mov     switch,ax
	mov     ax,1999                 ;set operation ok status
	sti
	iret                            ;dump timer irq

no_check2:
	cmp     ah,0fdh                 ;is set time requested
	jnz     no_check                ;no, continue
	mov     blanktime,bx            ;yes, set time
	mov     ax,1999                 ;set operation ok status
	sti
	iret                            ;dump timer irq

no_check:
	push    ax                      ;save registers
	push    bx
	push    cx
	push    dx
	push    si
	push    di
	push    ds
	push    es

	mov     al,blank_on             ;is function on ?
	cmp     al,1
	jz      t_1
	jmp     t_exit                  ;no exit

t_1:    mov     al,isblank              ;is screen blank
	cmp     al,0
	jnz     t_exit                  ;no need to change pointer

	mov     ax,switch               ;increment switch
	inc     ax
	mov     switch,ax

	;cmp    ax,6000                 ;6000 = 5 minutes
	cmp     ax,blanktime            ;300 = 15 seconds
	jle     t_exit

	call    video_disable

	mov     al,1
	mov     isblank,al

t_exit: pop     es                      ;restore registers
	pop     ds
	pop     di
	pop     si
	pop     dx
	pop     cx
	pop     bx
	pop     ax
	sti
	jmp     old_timerint            ;jump to old INT 9

int_timer       endp

;****************************************************************************
;*
;*
;****************************************************************************
video_disable   proc    near
		cmp     adapter,4
		jge     dis_mcga_vga
		cmp     adapter,3
		jge     dis_pga
		cmp     adapter,2
		jge     dis_ega
		mov     ax,500
		mov     dx,msr_address          ;it's MDA/CGA/HERCULES
		mov     al,25h                  ;disable and return
		out     dx,al
		ret
dis_ega:
		mov     dx,03bah
		in      al,dx
		mov     dx,03dah
		in      al,dx
		mov     dx,03c0h
		xor     al,al
		out     dx,al
		ret
dis_pga:        call    dis_ega
		ret
dis_mcga_vga:   mov     dx,03c4h
		in      ax,dx
		mov     vga_index,ax
		mov     ax,1
		out     dx,ax
		mov     dx,03c5h
		in      ax,dx
		mov     vga_clock,ax
		mov     ax,32
		out     dx,ax
		ret
video_disable   endp

;****************************************************************************
;*
;*
;****************************************************************************
video_enable    proc    near
		mov     al,adapter
		cmp     al,4
		jge     ena_mcga_vga
		cmp     al,3
		jge     ena_pga
		cmp     al,2
		jge     ena_ega

		mov     dx,msr_address          ;it's MDA/CGA/HERCULES
		mov     ah,15
		int     10h
		lea     bx,msr_values
		xlat    msr_values
		out     dx,al
		ret
ena_ega:
		mov     dx,03bah
		in      al,dx
		mov     dx,03dah
		in      al,dx
		mov     dx,03c0h
		mov     al,20
		out     dx,al
		ret
ena_pga:        call ena_ega
		ret
ena_mcga_vga:   mov     dx,03c5h
		mov     ax,vga_clock
		out     dx,ax
		mov     dx,03c4h
		mov     ax,vga_index
		out     dx,ax
		ret
video_enable    endp


;****************************************************************************
;*
;*
;****************************************************************************
init    proc    near
	xor     ax,ax
	call    QAdapter
	cmp     ax,-1                   ;was it unknown
	jnz     disp_found
	mov     dx,offset error_unk     ;yes, tello user and exit
	mov     ah,09
	int     21H
	int     20h

disp_found:

	mov     ah,0ffh                 ;check if installed already
	int     1ch
	cmp     ax,1999
	jnz     no_prior                ;no continue
	mov     dx,offset error_ari     ;yes, tello user and exit
	mov     ah,09
	int     21H
	int     20h

no_prior:
	push    cs                      ;make DS=CS
	pop     ds

	mov     al,09H                  ;Set al the the vector we use
	mov     ah,35H                  ;Use DOS-function call to get
	int     21H                     ;it's address.

	mov     old_kbdvec,bx           ;Old vector is returned in ES:BX
	mov     old_kbdvec[2],es        ;save it to old_io.

	mov     bx,cs                   ;DOS function wants vector in DS:DX
        mov     ds,bx
	mov     dx,offset int_kbd
	mov     al,09H                  ;Use DOS function call to set it.
	mov     ah,25H
	int     21H

	push    cs                      ;make DS=CS
	pop     ds

	mov     al,1CH                  ;Set al the the vector we use
	mov     ah,35H                  ;Use DOS-function call to get
	int     21H                     ;it's address.

	mov     old_timervec,bx         ;Old vector is returned in ES:BX
	mov     old_timervec[2],es      ;save it to old_io.

	mov     bx,cs                   ;DOS function wants vector in DS:DX
	mov     ds,bx
	mov     dx,offset int_timer
	mov     al,1CH                  ;Use DOS function call to set it.
	mov     ah,25H
	int     21H

	mov     dx,offset sign_on       ;display sign_on message
        mov     ah,09
	int     21H

	mov     bx,offset init          ;set save area
	int     27H                     ;Terminate & Stay

init    endp

sign_on:        db      10,13
		db      'SaveScreen Ver 1.00 (MDA/CGA/HERCULES/EGA/MCGA/VGA)',10,13
		db      10,13
		db      'Copyright (C) 1987-89  Software Technologies',10,10,13
		db      '$'

error_ari:
		db      10,13,'SaveScreen already loaded !!!',10,13
		db      '$'

error_unk:      db      10,13,'Unknown display, SaveScreen cannot load!!!',10,13
		db      '$'

;****************************************************************************
;*
;*
;****************************************************************************

QAdapter        proc    near

		push    bx                      ;save registers
		push    cx
		push    dx
		push    si
		push    di
		push    ds
		push    es

		mov     ah,1ah                  ;Try to get display
		mov     al,00h                  ;combination code.
		int     10h
		cmp     al,1ah                  ; did it work
		jnz     no_psvideo              ; no, check for EGA
		cmp     bl,00h
		jz      no_display              ;no display
		cmp     bl,01h
		jle     mda_display             ;mono
		cmp     bl,03h
		jle     cga_display             ;color graphics
		cmp     bl,05h
		jle     ega_display             ;enhanced graphics
		cmp     bl,06h
		jz      pga_display             ;professional graphics
		cmp     bl,08h
		jle     vga_display             ;VGA
		cmp     bl,0ch
		jle     mcga_display            ;MCA
		jmp     no_display

no_psvideo:     mov     ah,12h                  ;return EGA info
		mov     bl,10h
		int     10h

		cmp     bl,10h
		jz      mda_or_cga
		jmp     ega_display


mda_or_cga:     mov     ax,40h                  ;Get adapter
		push    ax
		pop     es
		mov     ax,es:[63h]             ;save msr_address
		add     ax,4
		mov     msr_address,ax

		mov     al,es:[10h]
		and     al,110000b
		cmp     al,30h
		je      mda_display             ;It is MDA
		jmp     cga_display

no_display:     mov     ax,-1
		jmp     found_adapter
mda_display:    mov     ax,00
		jmp     found_adapter
cga_display:    mov     ax,01
		jmp     found_adapter
ega_display:    mov     ax,02
		jmp     found_adapter
pga_display:    mov     ax,03
		jmp     found_adapter
vga_display:    mov     ax,05
		jmp     found_adapter
mcga_display:   mov     ax,04

found_adapter:  mov     adapter,al              ;screen adapter type
		pop     es                      ;restore registers
		pop     ds
		pop     di
		pop     si
		pop     dx
		pop     cx
		pop     bx
		ret

QAdapter        endp

code    ends

        end     start
