
;******************************************************************************
;*          This is to be placed on a floppy disk on the 1:rst sector.
;*          That's the boot record.
;*          Which will be loaded into ram at address, 0000:7c00 hex.
;******************************************************************************

;********** Interrupts in use *************************************************

SCREEN      =   00010h
KEYBOARD    =   00016h
REBOOT      =   00019h

;********** Functions and parameters ******************************************

STACKSTART  =   07C00h
BW_80x25    =   00002h
GET_CRTMODE =   0000fh
BW_CARD     =   00007h
MONO_SEG    =   0b000h
COL_SEG     =   0b800h
CRT_MODEREG =   003d8h
MOV_CURSOR  =   00002h
WRITE_TTY   =   0000eh
PROMPT_POS  =   01544h

;********** Here the program starts *******************************************

pcode       segment
            assume  cs:pcode,ds:pcode

main        proc    far
            cld
            cli
            xor     ax,ax
            mov     ss,ax
            mov     sp,STACKSTART
            jmp     short continue
            db      000h,002h,002h,001h,000h,002h,070h,000h
            db      0D0h,002h,0FDh,002h,000h,009h,000h,002h
            db      000h,000h,000h
continue:
            mov     ds,ax
            sti
            mov     al,BW_80x25
            int     SCREEN
            mov     ah,GET_CRTMODE
            int     SCREEN
            mov     bx,MONO_SEG
            cmp     al,BW_CARD
            jz      short bnw_card
            mov     bx,COL_SEG
            mov     dx,CRT_MODEREG
            mov     al,01h
            out     dx,al
bnw_card:
            mov     es,bx
            mov     si,offset data + STACKSTART
            lodsw
            mov     cx,ax
frame_loop:
            push    cx
            lodsw
            mov     di,ax
            lodsw
            mov     bp,ax
            lodsw
            mov     cx,ax
            lodsw
char_loop:
            stosw
            add     di,bp
            loop    short char_loop
            pop     cx
            loop    short frame_loop
            lodsw
            mov     cx,ax
line_loop:
            push    cx
            lodsw
            mov     di,ax
            xor     cx,cx
            lodsw
            mov     cl,al
text_loop:
            push    cx
            lodsb
            stosw
            pop     cx
            loop    short text_loop
            pop     cx
            loop    short line_loop
            cmp     bx,COL_SEG
            jnz     short bw_screen
            mov     al,29h
            mov     dx,CRT_MODEREG
            out     dx,al
bw_screen:
            xor     bh,bh
            mov     dx,PROMPT_POS
            mov     ah,MOV_CURSOR
            int     SCREEN
            xor     ax,ax
            int     KEYBOARD
            mov     ax,BW_80x25
            int     SCREEN
            int     REBOOT
main        endp

;********** Data area *********************************************************

data        dw      08h                         ; frame item count
item_1      dw      0000h,000h,01h,0fc9h        ; offset,addval,count,attr+char
item_2      dw      009eh,000h,01h,0fbbh
item_3      dw      0f00h,000h,01h,0fc8h
item_4      dw      0f9eh,000h,01h,0fbch
item_5      dw      0002h,000h,4eh,0fcdh
item_6      dw      0f02h,000h,4eh,0fcdh
item_7      dw      00a0h,09eh,17h,0fbah
item_8      dw      013eh,09eh,17h,0fbah
            dw      8                                       ; message counter
            dw      033ch                                   ; message offset
            db      42,0fh                                  ; length,attribute
            db      'THIS DISK HASN',27h,'T GOT ANY OPERATINGSYSTEM !'
            dw      0524h
            db      35,07h
            db      'Contains either data files or prog-'
            dw      05c4h
            db      34,07h
            db      'rams. Programs should be copied to'
            dw      0664h
            db      31,07h
            db      'another disk before actual use!'
            dw      07a4h
            db      21,07h
            db      'This disk comes from:'
            dw      0844h
            db      30,07h
            db      'XXXXXXXXXXXXXXXXXXXXXXXXX'
            dw      08e4h
            db      33,07h
            db      'XXXXXXXXXXXXXXXXXXXXXXXXX'
            dw      0d44h
            db      49,0fh
            db      'Insert a diskette with DOS.  Then press any key :'
pcode       ends
            end     main
