;****************************************************
;* FFORMAT: Program for fast formatting.            *
;****************************************************
;This program fast-formats a disk in 33 sec.
;
;Use dos SYS.COM to create system disk

;***************************************************

code    segment                 ;define code segment

;----------------------------------------------------

        assume      cs:code,ds:code

        org         100h        ;first address


main    proc    near

        mov     si,0081h         ;get drive specifier
        lodsb
        cmp     al,13            ;see if end of parameter
        je      nodrive          ;no drive
        dec     si
        mov     ch,00
        mov     cl,byte ptr [si-1]  ;Get length of cmdline
it:     cmp     byte ptr [si+1],':' ;Search for :
        jz      drivefound
        inc     si
        loop    it
        jmp     nodrive

drivefound:
        mov     al,byte ptr [si]
        and     al,0dfh
        cmp     al,'A'
        jz      adrive
        cmp     al,'B'
        jnz     nodrive

adrive: mov     di,OFFSET mess2
        mov     ah,':'
        mov     word ptr [di+41],ax

        sub     al,'A'
        mov     di,offset drive
        mov     [di],al
        jmp     start

nodrive:
        mov     dx,OFFSET mess7
        mov     ah,09
        int     21h
        int     20h



start:  mov     ax,0003h         ;Clear screen
        int     10h

        mov     dx,OFFSET mess1  ;Display PROGRAMNAME and
        mov     ah,09            ;Copyright message
        int     21h

        mov     dx,OFFSET mess2  ;Display message for start
        mov     ah,09
        int     21h

        mov     ah,08            ;Wait for key
        int     21h

        mov     dx,OFFSET mess3  ;Display message for
        mov     ah,09            ;formatting.
        int     21h


;       FORMAT LOOP
;---------------------------

        mov     di,OFFSET drive  ;Check for disk ready
        mov     dl,[di]
        mov     ah,00
        int     13h
        cmp     ah,00
        jnz     error

        push    cs               ;Set ES=CS
        pop     es
        mov     ch,39            ;Set start track number
next:   mov     di,OFFSET side0  ;This program starts at
        mov     ah,36            ;track 39d and goes
nxt0:   mov     [di],ch          ;downwards.
        sub     ah,04
        add     di,04
        cmp     ah,00
        jg      nxt0

        mov     di,OFFSET side0  ;Set for head 0
        inc     di
        mov     dh,00
        mov     ah,36
nxt1:   mov     [di],dh
        sub     ah,04
        add     di,04
        cmp     ah,00
        jg      nxt1

        call    format

        cmp     ah,00            ;Check for error
        jnz     error

        mov     di,OFFSET side0  ;Set for head 1
        inc     di
        mov     dh,01
        mov     ah,36
nxt2:   mov     [di],dh
        sub     ah,04
        add     di,04
        cmp     ah,00
        jg      nxt2

        call    format

        cmp     ah,00            ;Check for error
        jnz     error

        dec     ch               ;Check if entire disk is formatted
        cmp     ch,0ffh
        jnz     next
        jmp     dos

error:  mov     dx,OFFSET mess5  ;A error has occured, cant continue
        mov     ah,09            ;display message and
        int     21h
        int     20h              ;return to DOS

dos:    mov     dh,01            ;Set Directory on Side 1
        mov     cx,0001h
        mov     ax,0303h
        mov     bx,OFFSET sector
        int     13h
        cmp     ah,00
        jnz     error

        mov     dh,00            ;Write boot track
        mov     cx,0001h
        mov     ax,0301h
        mov     bx,OFFSET boot
        int     13h
        cmp     ah,00
        jnz     error

        mov     bx,OFFSET sector ;Set Directory on Side 0
        mov     cx,0005h
        mov     ax,0302h
        int     13h
        cmp     ah,00
        jnz     error

        mov     cx,0007h         ;More Directory on Side 0
        mov     ax,0303h
        int     13h
        cmp     ah,00
        jnz     error

        mov     cx,0003h         ;Set FAT blank part 1
        mov     ax,0301h
        int     13h
        cmp     ah,00
        jnz     error

        mov     di,OFFSET sector ;Set FAT information into sector
        mov     ah,0fdh
        mov     [di],ah
        mov     ah,0ffh
        inc     di
        mov     [di],ah
        inc     di
        mov     [di],ah

        mov     cx,0002h         ;Write FAT
        mov     ax,0301h
        int     13h
        cmp     ah,00
        jnz     error

        mov     cx,0004h         ;Write FAT-Copy
        mov     ax,0301h
        int     13h
        cmp     ah,00
        jnz     error

        mov     dx,OFFSET mess4  ;Format OK
        mov     ah,09            ;and
        int     21h

        mov     dx,OFFSET mess6  ;Ask for format
        mov     ah,09            ;again
        int     21h

ask:    mov     ah,08            ;Get key input
        int     21h

        cmp     al,'Y'           ;Check for Yes
        jz      format_again
        cmp     al,'y'
        jz      format_again
        cmp     al,'N'
        jz      exit_to_dos
        cmp     al,'n'
        jz      exit_to_dos
        jmp     ask

format_again:

        mov     bx,OFFSET sector ;Start format again
        mov     cx,0003h
clear:  mov     [bx],ch
        inc     bx
        loop    clear
        jmp     start

exit_to_dos:
        int     20H              ;return to DOS

format: mov     di,OFFSET drive
        mov     dl,[di]
        mov     cl,01
        mov     al,09
        mov     ah,05
        mov     bx,OFFSET side0
        int     13H              ;Format track via BIOS-INT 13H
        ret

main    endp

;---------------------------------------------------------
;       Table over strings
;---------------------------------------------------------

mess1   db      10,10,10,10,13
        db      'FAST-Format Ver 1.20,  '
        db      '1985,1986,1987$'

mess2   db      10,10,10,13,'Insert disk to be formatted in drive   '
        db      10,10,13,'Press any key when ready !$'

mess3   db      10,10,10,10,13,'Formating...$'

mess4   db      10,10,10,13,'Format complete$'

mess5   db      10,10,10,13,'Error while formating, can`t continue !!$'

mess6   db      10,10,13,'Format another (Y/N) ? $'

mess7   db      10,10,13,'Invalid drive specified',10,10,13,'$'

drive   db      00

side0   db      00,00,01,02,00,00,02,02,00,00,03,02
        db      00,00,04,02,00,00,05,02,00,00,06,02
        db      00,00,07,02,00,00,08,02,00,00,09,02

boot    db      00eh,01fh
        db      0B0h,003h,0B4h,000h,0CDh,010h,0BBh,01Dh,07Ch,0B9h,027h,000h
        db      08Ah,007h,0B4h,00Eh,0CDh,010h,043h,0E2h,0F7h,0B4h,000h,0CDh
        db      016h,0CDh,019h,049h,06Eh,073h,065h,072h,074h,020h,073h,079h
        db      073h,074h,065h,06Dh,020h,064h,069h,073h,06Bh,020h,061h,06Eh
        db      064h,020h,073h,074h,072h,069h,06Bh,065h,020h,061h,06Eh,079h
        db      020h,06Bh,065h,079h,02Eh

sector  db      1540 dup (0)

;---------------------------------------------------------

code    ends                     ;end of code segment
;*********************************************************


        end     main             ;end assembly
